
# q-expansion of newform 2400.2.d.f, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2400
weight = 2
poly_data = [8, -8, 6, -6, 3, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-2, 2, -1, 2, -1, 0], 1], [[8, -2, 4, -3, 0, -1], 2], [[-8, 10, -6, 3, -2, 1], 2], [[3, 0, 5, -1, 1, -1], 1], [[-12, 2, -6, 5, -2, 3], 2]]

hecke_ring_character_values = [[1951, [1, 0, 0, 0, 0, 0]], [901, [-1, 0, 0, 0, 0, 0]], [1601, [1, 0, 0, 0, 0, 0]], [577, [-1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -1], [0, 0, -1, 1, 0, 1], [0, -1, 0, 0, 0, 0], [0, 0, 1, 1, 0, 0], [0, 0, -1, 0, 0, 1], [0, 0, -1, 0, 0, 1], [0, 0, -1, 0, 0, 0], [2, 1, 0, 0, 0, 0], [-1, 0, 0, 0, 1, 0], [-3, -1, 0, 0, 1, 0], [-1, -1, 0, 0, 1, 0], [0, 0, -1, 2, 0, 1], [-2, 0, 0, 0, 0, 0], [0, 0, 1, 1, 0, 1], [0, 0, 2, -2, 0, -2], [4, 0, 0, 0, 0, 0], [1, -1, 0, 0, -1, 0], [0, 0, 3, 0, 0, 0], [6, -1, 0, 0, 0, 0], [5, 1, 0, 0, -1, 0], [5, -1, 0, 0, 1, 0], [0, 0, 3, 2, 0, 0], [0, 0, -1, 0, 0, 0], [0, 0, 0, 2, 0, -1], [7, -3, 0, 0, 1, 0], [0, 0, 0, 0, 0, 2], [0, 0, -3, -1, 0, -4], [0, 0, 4, 2, 0, -1], [0, 0, 1, -1, 0, -3], [0, 0, -5, 1, 0, 0], [0, 0, -5, -2, 0, 1], [0, 0, -1, 0, 0, 0], [-5, 2, 0, 0, -1, 0], [9, 2, 0, 0, -1, 0], [-7, 1, 0, 0, -1, 0], [0, 0, -3, 2, 0, 1], [-4, 0, 0, 0, 2, 0], [0, 0, -3, -1, 0, 1], [0, 0, -8, 0, 0, 2], [8, 0, 0, 0, 0, 0], [0, 0, -5, 2, 0, 4], [-14, -2, 0, 0, 0, 0], [-13, 2, 0, 0, -1, 0], [0, 0, -5, -2, 0, -3], [0, 0, -6, -2, 0, -1], [0, 2, 0, 0, 0, 0], [0, 0, -4, -2, 0, 2], [0, 0, 7, -1, 0, 0], [11, -1, 0, 0, 1, 0], [4, 0, 0, 0, -2, 0], [0, 0, -1, 3, 0, -3], [0, 0, 3, -3, 0, 0], [0, 0, -7, 2, 0, 1], [0, 0, 3, 0, 0, 4], [-21, -2, 0, 0, -1, 0], [-3, 4, 0, 0, -1, 0], [-4, 0, 0, 0, 2, 0], [20, 0, 0, 0, 0, 0], [-2, -4, 0, 0, 0, 0], [15, 3, 0, 0, 1, 0], [-7, 1, 0, 0, 3, 0], [0, 0, -3, 2, 0, 0], [10, 2, 0, 0, -4, 0], [0, 0, 7, -4, 0, -3], [0, 0, 3, -4, 0, -4], [-6, 0, 0, 0, -2, 0], [0, 0, 6, -2, 0, -6], [0, 0, 1, 1, 0, -4], [-1, 1, 0, 0, 1, 0], [0, 0, 2, -4, 0, -5], [-9, 0, 0, 0, 1, 0], [0, 0, -1, 2, 0, -3], [0, 0, 1, 0, 0, 1], [0, 0, -5, 4, 0, 0], [2, -1, 0, 0, 2, 0], [-5, -1, 0, 0, -1, 0], [6, 4, 0, 0, 0, 0], [0, 0, -1, 3, 0, 1], [0, 0, -12, -2, 0, 2], [1, -1, 0, 0, -1, 0], [0, 0, 1, -4, 0, 0], [1, 0, 0, 0, 1, 0], [-6, 4, 0, 0, -2, 0], [18, 4, 0, 0, 0, 0], [0, 0, 5, -2, 0, 4], [0, 0, 7, 0, 0, -4], [0, 0, 10, 0, 0, -1], [-19, -3, 0, 0, -1, 0], [-15, 3, 0, 0, -1, 0], [0, 0, 4, 2, 0, 3], [0, 0, -7, 1, 0, 1], [0, 0, 3, 4, 0, 1], [0, 0, -5, -2, 0, -3], [0, 0, 7, 4, 0, -8], [10, 0, 0, 0, 0, 0], [0, 0, 0, 0, -4, 0], [0, 0, 4, -4, 0, 2], [-5, -1, 0, 0, -3, 0], [16, 0, 0, 0, 2, 0], [-10, -4, 0, 0, 2, 0], [-15, 1, 0, 0, 1, 0], [0, 0, 7, 0, 0, 1], [0, 0, 3, -2, 0, -4], [9, 3, 0, 0, -1, 0], [0, 0, 1, -1, 0, 0], [15, 5, 0, 0, 1, 0], [-2, 0, 0, 0, 4, 0], [0, 0, 8, -4, 0, -1], [-4, 1, 0, 0, 0, 0], [0, 0, 9, -3, 0, 0], [0, 0, -9, -2, 0, 1], [16, 1, 0, 0, -2, 0], [-8, -4, 0, 0, -2, 0], [-1, -5, 0, 0, 1, 0], [0, 0, 3, -6, 0, -11], [-6, 2, 0, 0, -4, 0], [0, 0, -3, -3, 0, -7], [0, 0, -6, -2, 0, -6], [0, 0, 9, 0, 0, 0], [-6, 2, 0, 0, 4, 0], [-27, 1, 0, 0, -1, 0], [0, 0, -5, 0, 0, 5], [0, 0, 9, -4, 0, -4], [0, 0, -8, -2, 0, -2], [14, 0, 0, 0, -2, 0], [0, 0, 0, -6, 0, -9], [-9, -2, 0, 0, 5, 0], [0, 0, 11, -8, 0, -7], [0, 0, -11, -4, 0, 1], [-14, 5, 0, 0, 0, 0], [-21, -2, 0, 0, 1, 0], [4, -6, 0, 0, -2, 0], [-2, -6, 0, 0, 0, 0], [-2, -2, 0, 0, 0, 0], [10, 6, 0, 0, 2, 0], [10, 0, 0, 0, 0, 0], [-16, 8, 0, 0, -2, 0], [0, 0, -5, 6, 0, -3], [0, 0, -13, -4, 0, 4], [0, 0, -2, 6, 0, 3], [20, 0, 0, 0, 0, 0], [0, 0, 16, 0, 0, 2], [-11, 3, 0, 0, -5, 0], [8, -3, 0, 0, -4, 0], [0, 0, 3, 3, 0, 4], [0, 0, 3, -2, 0, 5], [0, 0, 5, -6, 0, -7], [-3, -2, 0, 0, 3, 0], [-3, 7, 0, 0, 1, 0], [-7, 9, 0, 0, -1, 0], [0, 0, 5, 6, 0, 5], [-14, -6, 0, 0, 2, 0], [-5, 7, 0, 0, 1, 0], [28, -1, 0, 0, -2, 0], [12, -2, 0, 0, -2, 0], [0, 0, 7, 0, 0, -12], [0, 0, 7, 0, 0, 4], [4, -4, 0, 0, 0, 0], [0, 0, -11, 3, 0, -4], [0, 0, 6, -2, 0, 7], [0, 0, -9, -5, 0, 5], [0, 0, -3, 1, 0, -4], [0, 0, 1, 0, 0, -11], [11, -4, 0, 0, 3, 0], [-4, -7, 0, 0, 0, 0], [22, -2, 0, 0, 4, 0], [14, -4, 0, 0, 4, 0], [0, 0, 5, -7, 0, 1], [0, 0, -6, 12, 0, 6], [-12, 12, 0, 0, -4, 0], [0, 0, 5, 2, 0, 4], [28, -3, 0, 0, 2, 0], [19, -3, 0, 0, 3, 0], [0, 0, 3, 2, 0, -7], [0, 0, -19, 0, 0, 4], [0, 0, 4, 0, 0, 3], [0, 0, -18, 4, 0, 2], [0, 0, -6, 2, 0, -5], [0, 0, -1, -3, 0, -7], [12, -1, 0, 0, -4, 0], [0, 0, -3, 7, 0, 0], [0, 0, 21, 0, 0, 5], [0, 0, -13, -4, 0, 4], [41, -4, 0, 0, 3, 0], [3, -1, 0, 0, -3, 0], [0, -10, 0, 0, 2, 0], [6, -2, 0, 0, 2, 0], [0, 0, 3, -6, 0, -8], [-18, -2, 0, 0, 6, 0], [0, 0, -5, -2, 0, -7], [0, 0, 19, 0, 0, 0], [-5, -3, 0, 0, 5, 0], [0, 0, -1, 5, 0, 4], [18, -4, 0, 0, 4, 0], [-22, 9, 0, 0, -2, 0], [0, 0, 15, -3, 0, -8], [0, 0, -23, 4, 0, 1], [0, 0, -7, 8, 0, 12], [-16, -3, 0, 0, 6, 0], [2, -3, 0, 0, -2, 0], [-8, 4, 0, 0, -6, 0], [0, 0, -13, -1, 0, 5], [18, 8, 0, 0, 4, 0], [-2, -7, 0, 0, -4, 0], [-2, -4, 0, 0, -6, 0], [-5, -3, 0, 0, -5, 0], [0, 0, -1, 0, 0, 9], [0, 0, 5, -6, 0, 4], [0, 0, 5, 4, 0, -8], [0, 0, 28, 2, 0, -1], [-22, -2, 0, 0, 2, 0], [18, -8, 0, 0, 2, 0], [4, -8, 0, 0, 6, 0], [0, 0, -2, 4, 0, 3], [-3, 5, 0, 0, -3, 0], [0, 0, -7, 0, 0, 9], [18, -12, 0, 0, 4, 0], [0, 0, 0, 2, 0, -10], [-21, 2, 0, 0, -1, 0], [5, -9, 0, 0, 1, 0], [0, 0, 2, -8, 0, -9], [15, 7, 0, 0, -3, 0], [0, 0, -10, 4, 0, 10], [0, 0, 7, 1, 0, -16], [-32, 2, 0, 0, -4, 0], [0, 0, 6, 0, 0, -1], [0, 0, -3, -5, 0, -7], [46, -3, 0, 0, 2, 0], [0, 0, -9, -2, 0, -7], [-13, -6, 0, 0, -1, 0], [2, 2, 0, 0, 4, 0], [-50, -6, 0, 0, -2, 0], [0, 0, 3, -4, 0, -7], [16, 8, 0, 0, 2, 0], [14, 6, 0, 0, -4, 0], [0, 0, 7, 3, 0, -11], [-19, 11, 0, 0, -5, 0], [-12, 6, 0, 0, -4, 0], [0, 0, 11, 8, 0, 9], [0, 0, 14, -10, 0, -1], [0, 0, -2, -2, 0, -10], [0, 0, -19, 1, 0, 8], [44, 6, 0, 0, 0, 0], [0, 0, -2, -4, 0, 3], [0, 0, -19, 1, 0, 1], [0, 0, -13, -4, 0, -3], [0, 0, 9, 6, 0, 5], [28, -13, 0, 0, 4, 0], [-6, -4, 0, 0, 0, 0], [0, 0, -1, 8, 0, -3], [20, 4, 0, 0, 6, 0], [-4, 0, 0, 0, -2, 0], [0, 0, -23, -3, 0, 1], [0, 0, -12, 0, 0, -10], [7, -5, 0, 0, 1, 0], [-24, 4, 0, 0, -6, 0], [0, 0, -15, -4, 0, -12], [0, 0, 6, -10, 0, -17], [8, 4, 0, 0, -4, 0], [0, 0, -16, -2, 0, 2], [-49, -4, 0, 0, 1, 0], [0, 0, 21, 3, 0, 4], [0, 0, 15, 4, 0, 1], [0, 0, 9, 4, 0, -4], [-43, 1, 0, 0, -3, 0], [2, 6, 0, 0, 2, 0], [-30, -4, 0, 0, 0, 0], [0, 0, -2, -8, 0, -2], [-27, -3, 0, 0, -5, 0], [0, 0, 5, 2, 0, 4], [18, 3, 0, 0, -4, 0], [0, 0, 13, 2, 0, 8], [0, 0, 4, 6, 0, -9], [-16, -2, 0, 0, 0, 0], [0, 0, 6, 2, 0, 2], [4, 8, 0, 0, -2, 0], [0, 0, -15, -5, 0, 9], [0, 0, 3, 4, 0, 5], [17, 6, 0, 0, 5, 0], [0, 0, 11, -6, 0, 1], [0, 0, 18, 0, 0, -6], [36, -8, 0, 0, 0, 0], [22, 2, 0, 0, -6, 0], [0, 0, -5, 8, 0, -4], [-26, 2, 0, 0, 0, 0], [5, 2, 0, 0, 1, 0], [-23, -3, 0, 0, 1, 0], [0, 0, -17, -4, 0, 8], [5, 7, 0, 0, -5, 0], [0, 0, 3, -13, 0, 0], [0, 0, 9, 11, 0, 1], [17, 6, 0, 0, -9, 0], [0, 0, 5, 12, 0, 0], [-8, -3, 0, 0, -2, 0], [-40, 0, 0, 0, 6, 0], [0, 0, 31, 3, 0, 1], [-49, 3, 0, 0, 1, 0], [0, 0, 5, 10, 0, 8], [10, 12, 0, 0, 0, 0], [19, -2, 0, 0, -1, 0]]
